#ifndef GAME_PLAYER_HPP
#define GAME_PLAYER_HPP

#include <array>
#include <SFML/Audio/Sound.hpp>
#include <SFML/Graphics/ConvexShape.hpp>
#include <SFML/Graphics/Rect.hpp>
#include <SFML/Graphics/View.hpp>
#include <SFML/System/Vector2.hpp>

#include "game/Object.hpp"
#include "game/SkillType.hpp"

namespace game
{
	class Player : public Object
	{
		public:
			Player( Game& theGame, GameScene& theScene, World& theWorld );
			
			virtual void update();
			virtual void render( sf::RenderWindow& window );
			
			int getMaxHealth() const;
			int getDamage() const;
			sf::Vector2f getPosition() const;
			sf::FloatRect getCollisionRect() const;
			sf::View getView() const;
			
			void playEnemyHurt();
			
			std::array< int, Skill::Count > skills;
		
		private:
			sf::Sound hurtSound;
			sf::Sound enemyHurtSound;
			int damage;
			int invuln;
			sf::ConvexShape shape;
			sf::FloatRect coll;
			sf::Vector2f vel;
			int shoot;
			
			sf::View view;
			
			void updateView();
			void updateDirection();
			void updatePosition();
			void checkHurt();
			void checkShooting();
	};
}

#endif // GAME_PLAYER_HPP
