#include "game/Projectile.hpp"

#include <cmath>
#include <SFML/Graphics/RenderWindow.hpp>
#include <util/Math.hpp>

#include "Game.hpp"
#include "game/Player.hpp"
#include "scenes/GameScene.hpp"

namespace game
{
	Projectile::Projectile( Game& theGame, GameScene& theScene, World& theWorld, sf::Vector2f pos, float dir, float theSpeed, bool thePlayer, int theLife )
	   : Object::Object( theGame, theScene, theWorld ),
	     speed( theSpeed ),
	     player( thePlayer ),
	     life( theLife ),
	     age( 0 )
	{
		sound.setBuffer( * game.getSoundBuffer( "res/sound/shoot.wav" ) );
		sound.setVolume( 25 );
		sound.play();
		
		shape.setRadius( 2.f );
		shape.setOrigin( 2, 2 );
		shape.setPosition( pos );
		shape.setRotation( dir );
		shape.setPointCount( 8 );
		
		if ( player )
		{
			shape.setFillColor( sf::Color( 0, 128, 255 ) );
			shape.setOutlineColor( sf::Color( 0, 64, 128, 128 ) );
		}
		else
		{
			shape.setFillColor( sf::Color( 255, 32, 0 ) );
			shape.setOutlineColor( sf::Color( 128, 16, 0, 128 ) );
		}
		shape.setOutlineThickness( 0.5f );
	}
	
	void Projectile::update()
	{
		float dir = util::degreesToRadians( -shape.getRotation() - 90 );
		
		sf::Vector2f vel;
		vel.x = std::cos( dir ) * speed;
		vel.y = std::sin( -dir ) * speed;
		
		shape.move( vel );
		
		sf::View view = scene.getPlayer().getView();
		sf::FloatRect viewRect( view.getCenter().x - ( view.getSize().x / 2 ), view.getCenter().y - ( view.getSize().y / 2 ), view.getSize().x, view.getSize().y );
		
		constexpr int MAX_AGE = 50 * 3;
		if ( life <= 0 or ( ++age >= MAX_AGE and !viewRect.contains( shape.getPosition() ) ) )
		{
			world.removeObject( this );
		}
	}
	
	void Projectile::render( sf::RenderWindow& window )
	{
		window.draw( shape );
	}
	
	bool Projectile::isPlayer() const
	{
		return player;
	}
	
	sf::FloatRect Projectile::getCollisionRect() const
	{
		return shape.getGlobalBounds();
	}
	
	void Projectile::damage()
	{
		--life;
	}
	
	int Projectile::getLife() const
	{
		return life;
	}
}
