#include "game/World.hpp"

#include <algorithm>

namespace game
{
	void World::update()
	{
		for ( auto it = pendingRemoval.begin(); it != pendingRemoval.end(); ++it )
		{
			auto found = std::find( objects.begin(), objects.end(), ( * it ) );
			if ( found != objects.end() )
			{
				objects.erase( found );
			}
		}
		pendingRemoval.clear();
		
		for ( auto it = pendingAddition.begin(); it != pendingAddition.end(); ++it )
		{
			objects.push_back( * it );
		}
		pendingAddition.clear();
		
		for ( auto it = objects.begin(); it != objects.end(); ++it )
		{
			( * it )->update();
		}
	}

	void World::update( const sf::Event& event )
	{
		for ( auto it = objects.begin(); it != objects.end(); ++it )
		{
			( * it )->update( event );
		}
	}

	void World::render( sf::RenderWindow& window )
	{
		for ( auto it = objects.begin(); it != objects.end(); ++it )
		{
			( * it )->render( window );
		}
	}

	void World::addObject( Object* obj )
	{
		addObject( std::shared_ptr< Object >( obj ) );
	}

	void World::addObject( std::shared_ptr< Object > obj )
	{
		pendingAddition.push_back( obj );
	}

	void World::removeObject( Object* obj )
	{
		for ( auto it = objects.begin(); it != objects.end(); ++it )
		{
			if ( it->get() == obj )
			{
				removeObject( * it );
				return;
			}
		}
	}

	void World::removeObject( std::shared_ptr< Object > obj )
	{
		pendingRemoval.push_back( obj );
	}

	std::list< std::shared_ptr< Object > >& World::getObjects()
	{
		return objects;
	}

	const std::list< std::shared_ptr< Object > >& World::getObjects() const
	{
		return objects;
	}
	
	sf::FloatRect World::getBounds() const
	{
		return sf::FloatRect( -320, -240, 640 * 2, 480 * 2 );
	}
}
