#include "scenes/DifficultyScene.hpp"

#include <array>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <string>

#include "Game.hpp"
#include "game/Player.hpp"

DifficultyScene::DifficultyScene( Game& theGame, const std::string& theNextScene )
   : MenuScene::MenuScene( theGame ),
     nextScene( theNextScene )
{
	subtitle = ( nextScene == "Skills" ) ? "Game Difficulty" : "Highscores";
}

void DifficultyScene::renderImpl( sf::RenderWindow& window )
{
	sf::Vector2i mouse = sf::Mouse::getPosition( window );
	
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Texture& button = ( * game.getTexture( "res/button/rectangle.png" ) );
	
	std::array< std::string, 3 > diffs{ "Easy", "Normal", "Hard" };
	for ( int id = 0; id < 3; ++id )
	{
		float x = 320 - ( button.getSize().x / 2 );
		float y = 160 + ( id * 56 );
		
		sf::Sprite spr;
		spr.setTexture( button, true );
		spr.setPosition( x, y );
		window.draw( spr );
		
		sf::Text text;
		text.setFont( font );
		text.setString( diffs[ id ] );
		text.setColor( sf::Color::Black );
		text.setCharacterSize( 40 );
		text.setPosition( x + static_cast< int >( ( button.getSize().x - text.getLocalBounds().width ) / 2 ), y );
		window.draw( text );
		
		if ( justPressed and spr.getGlobalBounds().contains( mouse.x, mouse.y ) )
		{
			SceneChangeEvent event;
			event.difficulty = id;
			game.changeScenes( nextScene, event );
		}
	}
	
	sf::Sprite spr;
	spr.setTexture( button, true );
	spr.setPosition( 320 - 128, 350 );
	window.draw( spr );
	
	sf::Text text;
	text.setFont( font );
	text.setString( "Back" );
	text.setColor( sf::Color::Black );
	text.setCharacterSize( 35 );
	text.setPosition( 320 - 128 + static_cast< int >( ( button.getSize().x - text.getLocalBounds().width ) / 2 ), 350 );
	window.draw( text );
	
	if ( justPressed and spr.getGlobalBounds().contains( mouse.x, mouse.y ) )
	{
		game.changeScenes( "Menu" );
	}
}
