#include "scenes/HighscoreScene.hpp"

#include <fstream>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <util/String.hpp>

#include "Game.hpp"
#include "game/Player.hpp"

HighscoreScene::HighscoreScene( Game& theGame )
   : MenuScene::MenuScene( theGame )
{
}

void HighscoreScene::initialize( SceneChangeEvent& event )
{
	MenuScene::initialize( event );
	
	difficulty = event.difficulty;
	loadHighscores();
	
	std::array< std::string, 3 > diffs{ "Easy", "Normal", "Hard" };
	subtitle = "Highscores (" + util::toString( diffs[ difficulty ] ) + ")";
}


void HighscoreScene::renderImpl( sf::RenderWindow& window )
{
	sf::Vector2i mouse = sf::Mouse::getPosition( window );
	
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Texture& button = ( * game.getTexture( "res/button/rectangle.png" ) );
	
	for ( int is = 0; is < scores.size(); ++is )
	{
		sf::Text num;
		num.setFont( font );
		num.setColor( sf::Color::Black );
		num.setCharacterSize( 27 );
		num.setString( util::toString( is + 1 ) + "." );
		num.setPosition( 64, 115 + ( is * 30 ) );
		window.draw( num );
		
		sf::Text name = num;
		name.setString( scores[ is ].second );
		name.move( 50, 0 );
		window.draw( name );
		
		sf::Text score = name;
		score.setString( util::toString( scores[ is ].first ) );
		score.setPosition( 640 - 64 - score.getLocalBounds().width, num.getPosition().y );
		window.draw( score );
	}
	
	sf::Sprite spr;
	spr.setTexture( button, true );
	spr.setPosition( 320 - 128, 480 - 5 - 56 );
	window.draw( spr );
	
	sf::Text back;
	back.setFont( font );
	back.setColor( sf::Color::Black );
	back.setString( "Back" );
	back.setCharacterSize( 40 );
	back.setPosition( spr.getPosition() );
	back.move( 128 - ( back.getLocalBounds().width / 2 ), 0 );
	window.draw( back );
	
	if ( justPressed and spr.getGlobalBounds().contains( mouse.x, mouse.y ) )
	{
		game.changeScenes( "Menu" );
	}
}

void HighscoreScene::loadHighscores()
{
	for ( int i = 0; i < scores.size(); scores[ i++ ].first = 0 );
	
	do
	{
		std::ifstream in( "res/scores/" + util::toString( difficulty ) + ".dat" );
		if ( !in )
		{
			break;
		}
		
		for ( int i = 0; i < scores.size(); ++i )
		{
			std::string line;
			std::getline( in, line );
			
			if ( !in )
			{
				break;
			}
			
			std::size_t found = line.find( '=' );
			if ( found == std::string::npos )
			{
				--i;
				continue;
			}
			
			scores[ i ].first = util::fromString< int >( line.substr( 0, found ) );
			scores[ i ].second = line.substr( found + 1 );
		}
	}
	while ( false );
}
