#include "scenes/MainMenuScene.hpp"

#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Texture.hpp>

#include "Game.hpp"
#include "game/Player.hpp"

MainMenuScene::MainMenuScene( Game& theGame )
   : MenuScene::MenuScene( theGame )
{
}

void MainMenuScene::renderImpl( sf::RenderWindow& window )
{
	sf::Vector2i mouse = sf::Mouse::getPosition( window );
	
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Texture& button = ( * game.getTexture( "res/button/rectangle.png" ) );
	
	sf::Sprite play;
	play.setTexture( button, true );
	play.setPosition( 320 - 128, 160 );
	window.draw( play );
	
	sf::Text text;
	text.setColor( sf::Color::Black );
	text.setFont( font );
	text.setString( "Play" );
	text.setCharacterSize( 40 );
	text.setPosition( play.getPosition() );
	text.move( 128 - ( text.getLocalBounds().width / 2 ), 0 );
	window.draw( text );
	
	sf::Sprite highscores = play;
	highscores.move( 0, 64 );
	window.draw( highscores );
	
	text.setString( "Highscores" );
	text.setPosition( highscores.getPosition() );
	text.move( 128 - static_cast< int >( text.getLocalBounds().width / 2 ), 0 );
	window.draw( text );
	
	sf::Sprite quit = highscores;
	quit.move( 0, 88 );
	window.draw( quit );
	
	text.setString( "Quit" );
	text.setPosition( quit.getPosition() );
	text.move( 128 - static_cast< int >( text.getLocalBounds().width / 2 ), 0 );
	window.draw( text );
	
	if ( justPressed )
	{
		if ( play.getGlobalBounds().contains( mouse.x, mouse.y ) )
		{
			game.changeScenes( "Game Difficulty" );
		}
		else if ( highscores.getGlobalBounds().contains( mouse.x, mouse.y ) )
		{
			game.changeScenes( "Difficulty Scores" );
		}
		else if ( quit.getGlobalBounds().contains( mouse.x, mouse.y ) )
		{
			game.close();
		}
	}
}
