#include "scenes/MenuScene.hpp"

#include <fstream>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Text.hpp>
#include <util/String.hpp>

#include "Game.hpp"
#include "game/Player.hpp"

int MenuScene::titleTimer = 0;
sf::ConvexShape MenuScene::base;
std::vector< sf::ConvexShape > MenuScene::original;

MenuScene::MenuScene( Game& theGame )
   : Scene::Scene( theGame )
{
}

void MenuScene::initialize( SceneChangeEvent& event )
{
	justPressed = false;
	loadTitleData();
}

void MenuScene::terminate()
{
}

void MenuScene::update()
{
}

void MenuScene::update( const sf::Event& event )
{
	if ( event.type == sf::Event::MouseButtonPressed )
	{
		if ( event.mouseButton.button == sf::Mouse::Left )
		{
			justPressed = true;
		}
	}
	else if ( event.type == sf::Event::KeyPressed )
	{
		if ( event.key.code == sf::Keyboard::F5 )
		{
			loadTitleData();
		}
	}
}

void MenuScene::render( sf::RenderWindow& window )
{
	window.clear( sf::Color::White );
	
	drawTitle( window );
	renderImpl( window );
	
	justPressed = false;
}

void MenuScene::drawTitle( sf::RenderWindow& window )
{
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Font& fontI = ( * game.getFont( "res/dejavu/ttf/DejaVuSerif-Italic.ttf" ) );
	
	drawFancyTitle( window );
	
	sf::Text title;
	title.setFont( fontI );
	title.setString( "Swarm" );
	title.setColor( sf::Color::Black );
	title.setCharacterSize( 75 );
	title.setPosition( ( window.getSize().x - title.getLocalBounds().width ) / 2, 8 );
	if ( sf::Keyboard::isKeyPressed( sf::Keyboard::F1 ) ) window.draw( title );
	
	sf::Text subTitle = title;
	subTitle.setFont( font );
	subTitle.setString( subtitle );
	subTitle.setCharacterSize( 20 );
	subTitle.setPosition( static_cast< int >( ( 640 - subTitle.getLocalBounds().width ) / 2 ), 90 );
	window.draw( subTitle );
}

void MenuScene::loadTitleData()
{
	base = sf::ConvexShape();
	base.setPointCount( 4 );
	base.setPoint( 0, sf::Vector2f( 8, 18 ) );
	base.setPoint( 1, sf::Vector2f( 16, 0 ) );
	base.setPoint( 2, sf::Vector2f( 8, 4 ) );
	base.setPoint( 3, sf::Vector2f( 0, 0 ) );
	base.setOrigin( 8, 8 );
	base.setFillColor( sf::Color( 255, 32, 0 ) );
	base.setOutlineColor( sf::Color( 128, 16, 0, 128 ) );
	base.setOutlineThickness( 1.5f );
	
	std::ifstream file( "res/title.txt" );
	if ( !file )
	{
		return;
	}
	
	original.clear();
	while ( true )
	{
		std::string line;
		std::getline( file, line );
		if ( !file )
		{
			break;
		}
		
		std::vector< std::string > tokens = util::tokenize( line, " " );
		if ( tokens.size() != 3 ) continue;
		
		sf::ConvexShape copy = base;
		copy.setPosition( util::fromString< float >( tokens[ 0 ] ), util::fromString< float >( tokens[ 1 ] ) );
		copy.setRotation( util::fromString< float >( tokens[ 2 ] ) );
		
		original.push_back( copy );
	}
	curr = original;
}

void MenuScene::drawFancyTitle( sf::RenderWindow& window )
{
	sf::Transform transform;
	transform.translate( 185, 20 );
	
	static int prevI = 0;
	
	++titleTimer;
	for ( std::size_t i = 0; i < curr.size(); ++i )
	{
		if ( i >= titleTimer / 2 )
		{
			continue;
		}
		if ( titleTimer / 2 > prevI and prevI < curr.size() )
		{
			prevI = titleTimer / 2;
			
			if ( prevI % 2 == 0 )
			{
				titleSound.setVolume( 12 );
				titleSound.setPitch( 0.75f + ( ( rand() % 10 ) / 20.f ) );
				titleSound.setBuffer( * game.getSoundBuffer( "res/sound/title.wav" ) );
				titleSound.play();
			}
		}
		
		curr[ i ].setRotation( original[ i ].getRotation() + ( ( rand() % 20 ) - 10 ) );
		window.draw( curr[ i ], transform );
	}
}
