#include "scenes/SkillScene.hpp"

#include <array>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <string>
#include <util/String.hpp>

#include "Game.hpp"
#include "game/Player.hpp"

SkillScene::SkillScene( Game& theGame )
   : MenuScene::MenuScene( theGame )
{
}

void SkillScene::initialize( SceneChangeEvent& event )
{
	MenuScene::initialize( event );
	
	for ( int i = 0; i < game::Skill::Count; ++i )
	{
		skills[ i ] = 1;
	}
	std::array< int, 3 > amounts{ 12, 9, 7 };
	available = amounts[ event.difficulty ];
	pending = event;
	
	std::array< std::string, 3 > diffs{ "Easy", "Normal", "Hard" };
	subtitle = "Skills (" + diffs[ event.difficulty ] + ")";
}

void SkillScene::renderImpl( sf::RenderWindow& window )
{
	sf::Vector2i mouse = sf::Mouse::getPosition( window );
	
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Texture& button = ( * game.getTexture( "res/button/square.png" ) );
	sf::Texture& skill = ( * game.getTexture( "res/skill.png" ) );
	
	sf::Text availText;
	availText.setFont( font );
	availText.setColor( sf::Color::Black );
	availText.setString( "Available: " + util::toString( available ) );
	availText.setCharacterSize( 20 );
	availText.setPosition( ( window.getSize().x - availText.getLocalBounds().width ) / 2, 315 );
	window.draw( availText );
	
	std::array< std::string, 4 > strs{ "Health", "Speed", "Reload", "Pierce" };
	
	for ( int is = 0; is < game::Skill::Count; ++is )
	{
		float x1 = 320 - 16 - ( skill.getSize().x * 2.5 ) - 8 - button.getSize().x;
		float x2 = 320 + 16 + ( skill.getSize().x * 2.5 ) + 8;
		float y = 150 + ( ( 8 + 32 ) * is );
		
		if ( true )// mouse.y >= y and mouse.y < y + button.getSize().y )
		{
			x1 += 80;
			x2 += 80;
		
			sf::Text name;
			name.setFont( font );
			name.setString( strs[ is ] );
			name.setColor( sf::Color::Black );
			name.setCharacterSize( 25 );
			name.setPosition( 64/*x1 - 16 - name.getLocalBounds().width*/, y + 12 );
			window.draw( name );
		}
		
		sf::Sprite sprL, sprR;
		sprL.setTexture( button, true ); sprR.setTexture( button, true );
		sprL.setPosition( x1, y ); sprR.setPosition( x2, y );
		window.draw( sprL ); window.draw( sprR );
		
		if ( justPressed )
		{
			if ( sprL.getGlobalBounds().contains( mouse.x, mouse.y ) and skills[ is ] > 1 )
			{
				--skills[ is ];
				++available;
			}
			else if ( sprR.getGlobalBounds().contains( mouse.x, mouse.y ) and skills[ is ] < 5 and available > 0 )
			{
				++skills[ is ];
				--available;
			}
		}
		
		sf::Text textL, textR;
		textL.setFont( font ); textR.setFont( font );
		textL.setString( "-" ); textR.setString( "+" );
		textL.setColor( sf::Color::Black ); textR.setColor( sf::Color::Black );
		textL.setCharacterSize( 25 ); textR.setCharacterSize( 25 );
		textL.setPosition( x1, y ); textR.setPosition( x2, y );
		textL.move( 16 - static_cast< int >( textL.getLocalBounds().width / 2 ), 0 );
		textR.move( 16 - static_cast< int >( textR.getLocalBounds().width / 2 ), 0 );
		window.draw( textL ); window.draw( textR );
		
		for ( int il = 0; il < 5; ++il )
		{
			int x = x1 + 32 + 8 + ( 8 * il ) + ( skill.getSize().x * il );
			x += 8; // Temp fix
			
			sf::Sprite spr;
			spr.setTexture( skill );
			spr.setTextureRect( sf::IntRect( ( il >= skills[ is ] ) ? ( skill.getSize().x / 2 ) : 0, 0, skill.getSize().x / 2, skill.getSize().y ) );
			spr.setPosition( x, y );
			window.draw( spr );
		}
	}
	
	sf::Texture& button2 = ( * game.getTexture( "res/button/rectangle.png" ) );
	sf::Sprite doneSpr;
	doneSpr.setTexture( button2, true );
	doneSpr.setPosition( 320 - ( button2.getSize().x / 2 ), 480 - 64 );
	window.draw( doneSpr );
	
	sf::Text doneText;
	doneText.setFont( font );
	doneText.setString( "10 Seconds" );
	doneText.setColor( sf::Color::Black );
	doneText.setCharacterSize( 35 );
	doneText.setPosition( doneSpr.getPosition() );
	doneText.move( static_cast< int >( ( button2.getSize().x - doneText.getLocalBounds().width ) / 2 ), 3 );
	window.draw( doneText ); 
	
	if ( justPressed and doneSpr.getGlobalBounds().contains( mouse.x, mouse.y ) )
	{
		pending.skills = skills;
		game.changeScenes( "Game", pending );
	}
}
