#include "scenes/SubmitScoreScene.hpp"

#include <array>
#include <fstream>
#include <iostream>
#include <SFML/Graphics/Font.hpp>
#include <SFML/Graphics/RectangleShape.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Sprite.hpp>
#include <SFML/Graphics/Text.hpp>
#include <SFML/Graphics/Texture.hpp>
#include <string>
#include <util/String.hpp>

#include "Game.hpp"
#include "game/Player.hpp"

SubmitScoreScene::SubmitScoreScene( Game& theGame )
   : MenuScene::MenuScene( theGame )
{
}

void SubmitScoreScene::initialize( SceneChangeEvent& event )
{
	MenuScene::initialize( event );
	received = event;
	
	std::array< std::string, 3 > diffs{ "Easy", "Normal", "Hard" };
	subtitle = "You Win (" + diffs[ event.difficulty ] + ")";
}

void SubmitScoreScene::update( const sf::Event& event )
{
	MenuScene::update( event );
	
	if ( event.type == sf::Event::TextEntered )
	{
		if ( event.text.unicode == 8 )
		{
			if ( name.getSize() > 0 )
			{
				name.erase( name.getSize() - 1, 1 );
			}
		}
		else if ( event.text.unicode == 13 )
		{
			submit();
		}
		else
		{
			name += event.text.unicode;
		}
	}
}

void SubmitScoreScene::renderImpl( sf::RenderWindow& window )
{
	sf::Vector2i mouse = sf::Mouse::getPosition( window );
	
	sf::Font& font = ( * game.getFont( "res/dejavu/ttf/DejaVuSans.ttf" ) );
	sf::Texture& button = ( * game.getTexture( "res/button/rectangle.png" ) );
	
	sf::Text score;
	score.setFont( font );
	score.setColor( sf::Color::Black );
	score.setCharacterSize( 20 );
	score.setString( "Score: " + util::toString( received.score ) );
	score.setPosition( 320 - static_cast< int >( score.getLocalBounds().width / 2 ), 220 );
	window.draw( score );
	
	sf::RectangleShape shape;
	shape.setFillColor( sf::Color( 0, 0, 0, 32 ) );
	shape.setOutlineColor( sf::Color::Black );
	shape.setOutlineThickness( 3.f );
	shape.setPosition( 80, 250 );
	shape.setSize( sf::Vector2f( 480, 64 ) );
	window.draw( shape );
	
	sf::Text field = score;
	field.setString( name );
	field.setCharacterSize( 54 );
	field.setPosition( shape.getPosition() );
	field.move( 5, 5 );
	window.draw( field );
	
	sf::Sprite spr;
	spr.setTexture( button, true );
	spr.setPosition( 320 - 128, 420 );
	window.draw( spr );
	
	sf::Text submit = field;
	submit.setCharacterSize( 40 );
	submit.setString( "Submit" );
	submit.setPosition( spr.getPosition() );
	submit.move( 128 - static_cast< int >( submit.getLocalBounds().width / 2 ), 0 );
	window.draw( submit );
	
	if ( justPressed and spr.getGlobalBounds().contains( mouse.x, mouse.y ) )
	{
		this->submit();
	}
}

void SubmitScoreScene::submit()
{
	std::array< std::pair< int, std::string >, 10 > scores;
	for ( int i = 0; i < scores.size(); scores[ i++ ].first = 0 );
	
	// Reading
	int putAt = -1;
	do
	{
		std::ifstream in( "res/scores/" + util::toString( received.difficulty ) + ".dat" );
		if ( !in )
		{
			putAt = 0;
			break;
		}
		
		for ( int i = 0; i < scores.size(); ++i )
		{
			std::string line;
			std::getline( in, line );
			
			if ( !in )
			{
				if ( putAt == -1 ) putAt = i;
				break;
			}
			
			std::size_t found = line.find( '=' );
			if ( found == std::string::npos )
			{
				--i;
				continue;
			}
			
			scores[ i ].first = util::fromString< int >( line.substr( 0, found ) );
			scores[ i ].second = line.substr( found + 1 );
			
			if ( putAt == -1 and scores[ i ].first < received.score )
			{
				putAt = i;
			}
		}
	}
	while ( false );
	
	// Inserting
	if ( putAt >= 0 and putAt < scores.size() )
	{
		for ( int i = scores.size() - 1; i > putAt and i > 0; --i )
		{
			scores[ i ] = scores[ i - 1 ];
		}
		
		scores[ putAt ].first = received.score;
		scores[ putAt ].second = name;
	}
	
	// Writing
	do
	{
		std::ofstream out( "res/scores/" + util::toString( received.difficulty ) + ".dat", std::ofstream::trunc );
		if ( !out )
		{
			std::cout  << "Failed to open/create highscores (" << received.difficulty << ") file." << std::endl;
			break;
		}
		
		for ( int i = 0; i < scores.size(); ++i )
		{
			if ( scores[ i ].first <= 0 )
			{
				continue;
			}
			
			out << scores[ i ].first << "=" << scores[ i ].second << std::endl;
		}
	}
	while ( false );
	
	game.changeScenes( "Highscores", received );
}
